/*
** interface.c - misc interface functions
**
** Keith Zaback (krz@wcug.wwu.edu) Apr 98
*/

/* includes */
#include "ipxfury.h"
#include <string.h>
#include <stdlib.h>

/* extern */
extern GLOBALS	ifg;

/* protos */
INTERFACE 	*interface_find_name(char *iname);
INTERFACE 	*interface_find(int target, int type);
void		interfaces_info(void);
void		interface_setup(void);


/* globals */
char	*state_str[] = {
	"Initial",		/* 0 */
	"Starting",		/* 1 */
	"Closed",		/* 2 */
	"Stopped",		/* 3 */
	"Closing",		/* 4 */
	"Stopping",		/* 5 */
	"Req-Sent",		/* 6 */
	"Ack-Rcvd",		/* 7 */
	"Ack-Sent",		/* 8 */
	"Opened",		/* 9 */
};

/*
**	INTERFACE_FIND_NAME - returns a the named interface
*/
INTERFACE *interface_find_name(char *iname)
{
	int		i;
	INTERFACE	*iface;

	/* check for NULL */
	if( iname == NULL )
		return(NULL);
	
	/* start */
	iface = ifg.interfaces;
	
	/* just look thru the table */
	for( i = 0; i < ifg.total_interfaces; i++ ) {
		if( strcasecmp(iname, iface->name) == 0 ) {
			/* found it */
			return(iface);
		}
		
		iface++;
	}
	
	/* didn't find it */
	return(NULL);
}



/*
**	INTERFACE_FIND - returns the nth (target) device of 'type'
*/
INTERFACE *interface_find(int target, int type)
{
	int		i;
	INTERFACE	*iface;
	
	/* starting position */
	iface = ifg.interfaces;
	
	/* try to locate it */
	for( i = 0; i < ifg.total_interfaces; i++ ) {
		if( iface->type == type ) {
			target--;		/* when target reaches zero, that's us */
			if( target <= 0 ) {
				return(iface);
			}
		}
		/* next interface */
		iface++;
	}
	
	/* didn't make it */
	return(NULL);

}


/*
**	INTERFACES_INFO - prints info about the interfaces
*/
void interfaces_info()
{
	int		i;
	INTERFACE	*iface;
	
	/* starting place */
	iface = ifg.interfaces;
	
	/* for each one, display its info */
	for( i = 0; i < ifg.total_interfaces; i++ ) {
		notice("Interface (%d)", i);
		
		if( iface->type == FURY_IF_TYPE_SERIAL ) {
			if( iface->flags & FURY_IF_MODEM_LINES ) {
				notice(" Type: MODEM ");
			} else {
				notice(" Type: SERIAL");
			}
			notice("  Name: %-6s  Baud:  %-6d", iface->name, iface->data.s.s_baudrate);
			notice(" (%s)\n", state_str[iface->state]);
			
#ifdef FURY_ETHERNET			
		} else if ( iface->type == FURY_IF_TYPE_ETHER ) {
			notice(" Type: ETHER   Name: %-6s  Frame: ", iface->name);
			switch( iface->data.e.e_frametype ) {
				case FURY_FRAMETYPE_II:
					notice("EthII ");		break;
				case FURY_FRAMETYPE_802_3:
					notice("802.3 ");		break;
				case FURY_FRAMETYPE_802_2:
					notice("802.2 ");		break;
				default:
					notice("????  ");		break;
			}
			notice(" (%s)\n", state_str[iface->state]);
#endif /* FURY_ETHERNET */
			
		} else if( iface->type == FURY_IF_TYPE_UDP ) {
			notice(" Type: UDP     Port: %-6d               ", iface->data.u.u_portnumber);
			notice(" (%s)\n", state_str[iface->state]);
			
		} else {
			errormsg(0, "interface_info", "unknown type (%d) for interface %d", iface->type, i);
		}
		
		/* next one */
		iface++;
		
	} /* for */

}




/*
**	INTERFACE_SETUP - just sets the function ptrs
*/
void interface_setup()
{
	int		i;
	INTERFACE	*iface;

	/* starting ptr */
	iface = ifg.interfaces;
	
	/* loop thru the array, setting the functions needed */
	for( i = 0; i < ifg.total_interfaces; i++ ) {
		
		if( iface->type == FURY_IF_TYPE_SERIAL ) {
			iface->open		= (int (*)(void *))serial_open;
			iface->close		= (int (*)(void *))serial_close;
			iface->read		= (int (*)(void *))serial_read;
			iface->process		= (int (*)(void *))serial_process;
			iface->write		= (int (*)(void *))serial_write;
			iface->queue_ipx_frame	= (int (*)(void *, void *, void *))serial_queue_ipx_frame;
			iface->timeout_handler	= (int (*)(void *))serial_timeout;
			iface->flush		= (int (*)(void *))serial_flush;
			iface->event_handler	= (int (*)(void *, int))fury_event;
			iface->prepare_device	= (int (*)(void *))serial_prepare_device;
			
			iface->ipxframe_processor = (int (*)(void *, void *))serial_dummy;
#ifdef FURY_ETHERNET								
		} else if( iface->type == FURY_IF_TYPE_ETHER ) {
			iface->open 		= (int (*)(void *))ether_open;
			iface->close		= (int (*)(void *))ether_close;
			iface->read		= (int (*)(void *))ether_read;
			iface->process		= (int (*)(void *))ether_process;
			iface->write		= (int (*)(void *))ether_write;
			iface->queue_ipx_frame	= (int (*)(void *, void *, void *))ether_queue_ipx_frame;
			iface->flush		= (int (*)(void *))ether_flush;
			iface->event_handler	= (int (*)(void *, int))ether_event;
			iface->prepare_device	= (int (*)(void *))ether_prepare_device;
			
			iface->timeout_handler	= (int (*)(void *))ether_dummy;
			iface->ipxframe_processor = (int (*)(void *, void *))ether_dummy;
#endif /* FURY_ETHERNET */
		
		} else if( iface->type == FURY_IF_TYPE_UDP ) {
		
			iface->open 		= (int (*)(void *))udp_open;
			iface->close		= (int (*)(void *))udp_close;
			iface->read		= (int (*)(void *))udp_read;
			iface->process		= (int (*)(void *))udp_process;
			iface->write		= (int (*)(void *))udp_write;
			iface->queue_ipx_frame	= (int (*)(void *, void *, void *))udp_queue_ipx_frame;
			iface->event_handler	= (int (*)(void *, int))udp_event;
			iface->prepare_device	= (int (*)(void *))udp_prepare_device;
			iface->ipxframe_processor = (int (*)(void *, void *))udp_ipxframe_processor;
			
			iface->timeout_handler	= (int (*)(void *))udp_dummy;
			iface->flush		= (int (*)(void *))udp_dummy;
				
		} else {
			errormsg(0, "interface_setup", "unknown type (%d) for interface %d", iface->type, i); 
		}
		
		/* next one */
		iface++;
	} /* for */
	
	/* done */
}

